'use strict';

var chalk = require('chalk');
var diff = require('jest-diff');
var stripAnsi = require('strip-ansi');

var _require = require('jest-snapshot'),
    toMatchSnapshot = _require.toMatchSnapshot;

var isAddition = function isAddition(line) {
  return (/^\+/.test(line)
  );
};

var isDeletion = function isDeletion(line) {
  return (/^-/.test(line)
  );
};

var removeGlamorClassNames = function removeGlamorClassNames(line) {
  return line.replace(/css-.+?( |")/, '').slice(1).trim();
};
var isClassNameAttribute = function isClassNameAttribute(line) {
  var previous = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var next = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';

  var containsGlamorClassName = /class(Name)?=".*css-.*"/.test(line);
  if (containsGlamorClassName) {
    // just because the line contains a glamor class name doesn't mean that
    // it doesn't also have some other change, so we'll remove everything
    // except the glamor class name and see if it's the same as the previous
    // or the next line. It's not really perfect, but it's less likely that
    // we'll show something in white that shouldn't be :)
    var lineRemoved = removeGlamorClassNames(line);
    var previousRemoved = removeGlamorClassNames(previous);
    var nextRemoved = removeGlamorClassNames(next);
    return lineRemoved === previousRemoved || lineRemoved === nextRemoved;
  }
  return containsGlamorClassName;
};
var isDataAttribute = function isDataAttribute(line) {
  return (/data-css-.*/.test(line)
  );
};
var isClassNameSelector = function isClassNameSelector(line) {
  return (/\.css-.*,/.test(line)
  );
};
var isDataSelector = function isDataSelector(line) {
  return (/\[data-css-.*\] {/.test(line)
  );
};

var isClassName = function isClassName(line, previous, next) {
  return (isAddition(line) || isDeletion(line)) && (isClassNameAttribute(line, previous, next) || isDataAttribute(line) || isClassNameSelector(line) || isDataSelector(line));
};

var colorize = function colorize(message) {
  var messageLines = message.split('\n');

  return messageLines.map(function (line, index) {
    var previous = messageLines[index - 1];
    var next = messageLines[index + 1];
    if (isClassName(line, previous, next)) {
      return chalk.white(line);
    }
    if (isAddition(line)) {
      return chalk.red(line);
    }
    if (isDeletion(line)) {
      return chalk.green(line);
    }
    return chalk.dim(line);
  }).join('\n');
};

var matcher = {
  toMatchSnapshotWithGlamor() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var result = toMatchSnapshot.apply(this, args);
    var message = void 0;

    if (!result.pass) {
      message = diff(result.expected, result.actual, {
        aAnnotation: 'Snapshot',
        bAnnotation: 'Received'
      });
      message = stripAnsi(message);
      message = colorize(message);
    }

    return { pass: result.pass, message };
  }
};

module.exports = matcher;